@@ -22,23 +22,24 @@ from pymatgen.util.io_utils import zopen
 from pymatgen.io.vaspio.vasp_input import VaspInput
 from pymatgen.io.smartio import read_structure
 from pymatgen.io.vaspio_set import MITVaspInputSet
+from pymatgen.serializers.json_coders import MSONable, PMGJSONDecoder
 
 from custodian.ansible.intepreter import Modder
 from custodian.ansible.actions import FileActions, DictActions
 from custodian.custodian import Job
 
 
-VASP_INPUT_FILES = set(["INCAR", "POSCAR", "POTCAR", "KPOINTS"])
+VASP_INPUT_FILES = {"INCAR", "POSCAR", "POTCAR", "KPOINTS"}
 
 VASP_OUTPUT_FILES = ['DOSCAR', 'INCAR', 'KPOINTS', 'POSCAR', 'PROCAR',
                      'vasprun.xml', 'CHGCAR', 'CHG', 'EIGENVAL', 'OSZICAR',
                      'WAVECAR', 'CONTCAR', 'IBZKPT', 'OUTCAR']
 
 
-class VaspJob(Job):
+class VaspJob(Job, MSONable):
     """
-    A basic vasp job. Just runs whatever is in the directory. But
-    conceivably can be a complex processing of inputs etc. with initialization.
+    A basic vasp job. Just runs whatever is in the directory. But conceivably
+     can be a complex processing of inputs etc. with initialization.
     """
 
     def __init__(self, vasp_command, output_file="vasp.out", suffix="",
@@ -81,10 +82,11 @@ class VaspJob(Job):
             settings_override:
                 An ansible style list of dict to override changes. For example,
                 to set ISTART=1 for subsequent runs and to copy the CONTCAR
-                to the POSCAR, you will provide
-                [{"dict": "INCAR", "action": {"_set": {"ISTART": 1}}},
-                 {"filename": "CONTCAR",
-                  "action": {"_file_copy": {"dest": "POSCAR"}}}]
+                to the POSCAR, you will provide::
+
+                    [{"dict": "INCAR", "action": {"_set": {"ISTART": 1}}},
+                     {"filename": "CONTCAR",
+                      "action": {"_file_copy": {"dest": "POSCAR"}}}]
         """
         self.vasp_command = vasp_command
         self.output_file = output_file
@@ -106,7 +108,8 @@ class VaspJob(Job):
                 except:
                     pass
             if num_structures != 1:
-                raise RuntimeError("{} structures found. Unable to continue.")
+                raise RuntimeError("{} structures found. Unable to continue."
+                                   .format(num_structures))
             else:
                 self.default_vis.write_input(struct, ".")
 
@@ -171,6 +174,27 @@ class VaspJob(Job):
                         {"filename": "CONTCAR",
                          "action": {"_file_copy": {"dest": "POSCAR"}}}])]
 
+    @property
+    def to_dict(self):
+        d = dict(vasp_command=self.vasp_command,
+                 output_file=self.output_file, suffix=self.suffix,
+                 final=self.final, gzipped=self.gzipped, backup=self.backup,
+                 default_vasp_input_set=self.default_vis.to_dict,
+                 settings_override=self.settings_override
+                 )
+        d["@module"] = self.__class__.__module__
+        d["@class"] = self.__class__.__name__
+        return d
+
+    @staticmethod
+    def from_dict(d):
+        vis = PMGJSONDecoder().process_decoded(d["default_vasp_input_set"])
+        return VaspJob(
+            vasp_command=d["vasp_command"], output_file=d["output_file"],
+            suffix=d["suffix"], final=d["final"], gzipped=d["gzipped"],
+            backup=d["backup"], default_vasp_input_set=vis,
+            settings_override=d["settings_override"])
+
 
 def gzip_directory(path):
     """
